/*
 * This file implements some often used ISP commands
 * for AVR microcontrollers
 * 
 * by Stephan Brwolf, Ilmenau 2013
 */

#define __AVRISP_C_229db984fe7b4d419c4c43c9ad0e2028 1
#include "avrisp.h"

#include <stdint.h>

static uint64_t initialized = 0;

int avrisp_isinitialized(void) {
  return ((initialized>0)&&(softspi_isinitialized()));
}

int avrisp_initialize(void) {
  if (initialized==0) {

    // do sth. here:
    if (softspi_initialize()<0) return -1;

    initialized=1;
    return 0;
  }
  initialized++;
  return 1;
}

int avrisp_finalize(void) {
  if (initialized > 0) {
    initialized--;
    if (initialized==0) {
      
      // undo sth. here:
      softspi_finalize();
      
      return 0;
    } else return 1;
  }
  return 2;
}

int avrisp_enable(void) {
  int result = 0;
  
  softspi_disable();
  usleep(softspi_getclock()<<1);
  softspi_enable();
  usleep(softspi_getclock()<<1);

  softspi_kick(0b10101100);
  softspi_kick(0b01010011);
  if (softspi_kick(0b00000000) != 0b01010011) result=0-__LINE__;
  softspi_kick(0b00000000);
  return result;
}

int avrisp_disable(void) {
  softspi_disable();
  return 0;
}



static uint8_t avrisp_readsignaturebyte(uint8_t addrbyte) {
  softspi_kick(0b00110000);
  softspi_kick(0b00000000);
  softspi_kick(addrbyte&0x3);
  return softspi_kick(0b00000000);
}
uint32_t avrisp_readsignature(void) {
  uint32_t result	= 0;
  uint8_t  i	  	= 4;
  do {
    i--;
    result<<=8;
    result|=avrisp_readsignaturebyte(i);
  } while (i>0);

  return result;
}


static uint8_t avrisp_readcalibrationbyte(uint8_t addrbyte) {
  softspi_kick(0b00111000);
  softspi_kick(0b00000000);
  softspi_kick(addrbyte&0x3);
  return softspi_kick(0b00000000);
}
uint32_t avrisp_readcalibration(void) {
  uint32_t result	= 0;
  uint8_t  i	  	= 4;
  do {
    i--;
    result<<=8;
    result|=avrisp_readcalibrationbyte(i);
  } while (i>0);

  return result;
}



uint8_t avrisp_readlock(void) {
  softspi_kick(0b10101000);
  softspi_kick(0b00000000);
  softspi_kick(0b00000000);
  return softspi_kick(0b00000000);
}

uint8_t avrisp_readlfuse(void) {
  softspi_kick(0b01010000);
  softspi_kick(0b00000000);
  softspi_kick(0b00000000);
  return softspi_kick(0b00000000);
}

uint8_t avrisp_readhfuse(void) {
  softspi_kick(0b01011000);
  softspi_kick(0b00001000);
  softspi_kick(0b00000000);
  return softspi_kick(0b00000000);
}

uint8_t avrisp_readeeprom(uint16_t byteaddress) {
  softspi_kick(0b10100000);
  softspi_kick(byteaddress>>8);
  softspi_kick(byteaddress&0xff);
  return softspi_kick(0b00000000);
}

uint8_t avrisp_readflash(uint16_t byteaddress) {
  if (byteaddress & 0x1)	softspi_kick(0x28);
  else 				softspi_kick(0x20);
  byteaddress>>=1;
  softspi_kick(byteaddress>>8);
  softspi_kick(byteaddress&0xff);
  return softspi_kick(0x00);
}





void avrisp_chiperase(void) {
  softspi_kick(0b10101100);
  softspi_kick(0b10000000);
  softspi_kick(0b00000000);
  softspi_kick(0b00000000);
}

void avrisp_writelock(uint8_t data) {
  softspi_kick(0b10101100);
  softspi_kick(0b11100000);
  softspi_kick(0b00000000);
  softspi_kick(data);
}

void avrisp_writelfuse(uint8_t data) {
  softspi_kick(0b10101100);
  softspi_kick(0b10100000);
  softspi_kick(0b00000000);
  softspi_kick(data);
}

void avrisp_writehfuse(uint8_t data) {
  softspi_kick(0b10101100);
  softspi_kick(0b10101000);
  softspi_kick(0b00000000);
  softspi_kick(data);
}

void avrisp_writeeprom(uint16_t byteaddress, uint8_t data) {
  softspi_kick(0b11000000);
  softspi_kick(byteaddress>>8);
  softspi_kick(byteaddress&0xff);
  softspi_kick(data);
}

void avrisp_loadflashpage(uint16_t byteaddress, uint8_t data) {
  if (byteaddress & 0x1)	softspi_kick(0b01001000);
  else 				softspi_kick(0b01000000);
  byteaddress>>=1;
  softspi_kick(byteaddress>>8);
  softspi_kick(byteaddress&0xff);
  softspi_kick(data);
}

void avrisp_writeflashpage(uint16_t pagestartbyteaddress) {
  pagestartbyteaddress>>=1;
  softspi_kick(0b01001100);
  softspi_kick(pagestartbyteaddress>>8);
  softspi_kick(pagestartbyteaddress&0xff);
  softspi_kick(0b00000000);
}

